#! /usr/bin/python3
#

import os
import sys
from math import log
from subprocess import Popen, PIPE
from py3dephell.py3prov import generate_provides


def mkset(path):
    bpp = log(len(path), 2) + 11

    mksetP = Popen(['/usr/lib/rpm/mkset', '%d' % bpp],
                   stdin=PIPE, stdout=PIPE, universal_newlines=True)

    return mksetP.communicate(input='\n'.join(path))[0].strip()


def _check_duplicate_provides(nonstd_dict, std_dict):
    for nskey, nsvalue in nonstd_dict.items():
        for skey, svalue in std_dict.items():
            if svalue == nsvalue:
                print(f'python3.prov.py: {svalue} is provided by non-std path({nskey}) and'
                      f' std-path ({skey}), but the second one will be dropped.'
                      ' Try to separate your package', file=sys.stderr)


def main(files, rpm_path=[], verbose=False):
    buildroot = os.getenv('RPM_BUILD_ROOT', '')
    prefixes = sys.path[1:] + rpm_path
    prefixes = [os.path.normpath(f'{buildroot}/{p}' if not p.startswith(buildroot) else p) for p in prefixes]

    # To debug, where can we lost <0 provides
    non_std_path_provs = {}
    std_path_provs = {}
    for file, provides in generate_provides(files, prefixes, verbose=bool(verbose),
                                            abs_mode=True, only_prefix=True).items():
        provides = provides['provides']
        if provides:
            rpath = max([path for path in rpm_path if path in file and path],
                        key=lambda p: len(p.split('/')), default='')
            if rpath:
                set_v = mkset([rpath])
                for prov in provides:
                    if not prov.split('.')[-1] == '__init__':
                        pprov = f'python{sys.version_info[0]}({prov})'
                        print(f'{file} provides {pprov}'
                              f'(under non-std path {rpath})', file=sys.stderr)
                        print(f'{pprov} = {set_v}')
                        non_std_path_provs[file] = prov

            else:
                for prov in provides:
                    if not prov.split('.')[-1] == '__init__':
                        print(f'python{sys.version_info[0]}({prov})')
                        std_path_provs[file] = prov
        elif verbose:
            print(f'py3prov:{file} provides nothing', file=sys.stderr)

    _check_duplicate_provides(non_std_path_provs, std_path_provs)


if __name__ == '__main__':
    files = sys.argv[1:] or [x.strip() for x in sys.stdin.readlines()]

    br = os.path.normpath(os.getenv('RPM_BUILD_ROOT', ''))
    rpm_path = os.getenv('RPM_PYTHON3_PATH', '').split()
    rpm_path += os.getenv('RPM_PYTHON3_COMPILE_INCLUDE', '').split()
    # Sometimes (when we build python3) we can use custom python3 and modify
    # sys.path so that each path from it is prefixed with %%buildroot.
    rpm_path = set(rpm_path).difference(map(lambda p: p.removeprefix(br), sys.path[1:]))
    verbosity = int(os.getenv('RPM_SCRIPTS_DEBUG', 0))

    main(files, verbose=bool(verbosity))
    [main(files, rpm_path=[r_path], verbose=bool(verbosity)) for r_path in rpm_path if r_path]

# ex: set ts=8 sts=4 sw=4 et:
