#! /usr/bin/python3

import os
import sys
import io
import re
import shlex
from py3dephell.py3req import generate_requirements


def ignore():
    from_spec = os.getenv('RPM_PYTHON3_REQ_SKIP', "").split()
    builtin = list(sys.builtin_module_names) + ['__main__']
    return from_spec + builtin


REQ = os.getenv('RPM_PYTHON3_REQ_METHOD', 'strict')
IS_HIER = os.getenv('RPM_PYTHON3_REQ_HIER', None)
if IS_HIER and IS_HIER.startswith('no') != 0:
    # rpm-build-python/tools/requires.py uses 'hier' as the default value;
    # rpm-build-python3/python3 can set it to 'yes'.
    # So, let's suppose the opposite value can be either 'nohier' or 'no'
    # (or simply an unset env variable). Here we map those possibilities
    # to our internal canonical representation of this value
    # (just in case someone may use them).
    IS_HIER = None

buildroot = os.path.normpath(os.getenv('RPM_BUILD_ROOT', ''))

if len(sys.argv) > 1:
    file_list = sys.argv[1:]
else:
    file_list = shlex.split(sys.stdin.read())

if (add_provides := os.getenv('RPM_PYTHON3_SELF_PROV_PATH', [])):
    add_provides = [prov for prov in add_provides.split(':') if prov]

for file_name in file_list.copy():
    path, tail = os.path.splitext(file_name)
    if tail in ['.pyc', '.pyo', '.pth']:
        file_list.remove(file_name)

stderr = io.StringIO()
prefixes = list(map(lambda p: os.path.normpath(p if p.startswith(buildroot) else f'{buildroot}{p}'), sys.path))
dependencies = generate_requirements(files=file_list, add_prov_path=add_provides,
                                     prefixes=prefixes, ignore_list=ignore(),
                                     skip_subs=True, only_top_module=not IS_HIER,
                                     only_external_deps=REQ in ['slight', 'relaxed'],
                                     stderr=stderr, verbose=True)
for err in stderr.getvalue().split('\n'):
    err = re.sub('Ignore', f'Ignore for REQ:{REQ}', err)
    print(err, file=sys.stderr)

for file, deps in dependencies.items():
    print(*['\n'.join(reqs) for reqs in deps if reqs])
